#!/usr/bin/perl -w

######################################################
#
#	sensitivity wrapper
#
#	The sensitivity wrapper has the ability to produce a graph to show the sensity of the <output_result> to each parameter
#	for a given configuration. Each parameter are divided by 2 and 4 and multiplied by 2 and 4.
#
#	The results are printed in a file <output_file_name>.txt
#	and the graph is plot in a file <output_file_name>.eps
#
#	How to use it:
#	./sensitivity.pl	<dc config file>			datacenter configuration file
#						<resource config file>		resource configuration file
#						<output_file_name>			filename of the output file without extension (the extensions are .txt and .eps)
#						<output_result>				output parameter to analyze
#
#						[title:name]				Optional: to set the graph title 'name'. by default <output_result> is used
#						[nognuplot]					Optional: put "nognuplot" if you do not want to get the graph
#
#
#  Remarks:
#		The bounded parameters are not present: pvf, mttf_coef, coef_maintenance
#
#  Example of command:
# ./sensitivity.pl template/dc_template.params template/r_template.params tco tco title:"Normalized TCO"
#
######################################################

use strict;

use POSIX qw/ceil/;

use File::Copy;

use lib "../lib/"; 
use helper;


######################################################
#
# Get input parameters
#
######################################################

my $argv_size = scalar @ARGV;
if ($argv_size < 4 )
{
	print "usage: ./sensitivity.pl <dc config file> <resource config file> <output_file_name> <output_result> [title:name] [nognuplot]\n";
	exit(0);
}

my $dc_filename=$ARGV[0];
my $resource_filename=$ARGV[1];

my $filename= $ARGV[2];
my $output= $ARGV[3];

my $gnuplot=1;
my $title=$output;
for(my $i=4; $i<6; $i++)
{
	if(defined($ARGV[$i]))
	{
		my $arg_tmp=$ARGV[$i];
		
		my @list=split(':',$arg_tmp);
		
		if($list[0] eq "nognuplot")
		{
			$gnuplot=0;
		}
		elsif($list[0] eq "title")
		{
			$title=$list[1];
		}
		else{print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
	}
	else{last;}
}

######################################################
#
# General Initialization
#
######################################################

#create the helper
my $helper = helper->new("");

#check that the output name is correct
if($helper->checkOutputValueName($output)==-1){print("Execution stopped\n");exit(0); }
if($helper->checkOutputValueName($output)==1){print("Warning: The parameter 'report_resources' in the DC file must be set to 1 to get correct results\n"); }

#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

#open outputfile
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;


######################################################
#
# Specific Initialization
#
######################################################


######################################################
#get the reference value
######################################################

my $ptr_dc=$helper->parserDCFile("$dc_filename");
my $ptr_resource=$helper->parserResourceFile("$resource_filename");

$helper->createParameterFile("$currentdir/dc.params",$ptr_dc);
$helper->createParameterFile("$currentdir/s.params",$ptr_resource);


$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/ref");

#get the result value
my $ptr_res=$helper->parserResultKernel("$currentdir/output/ref");
my %res=%{$ptr_res};

my $ref=$res{$output};


######################################################
#sensitivity values & list of parameters to consider
######################################################

my @sensitivity_values=(0.25,0.5,2,4);

my @list_parameters=(
"a_coolingarea",
"c_coolpowerdeplperW",
"salary_perrackpermonth",
"c_buildingpersqm",
"PUE",
"c_electperKWH",
"dcdepr",
#"cost_of_money",
#########
"c_system",
"k_chipspersrvmodule",
"mttf_dram",
"p_dram_idle",
"p_system_idle",
"p_switchport",
"p_dram",
#"mttf_coef",
"k_bladesperrack",
"p_switchport_idle",
"u",
"c_other",
"mttr",
"networkdepr",
"chip_perf",
"minimumdistance",
"SPUE",
"depth",
"pvf",
"c_switchport",
"srvdepr",
"c_networkperrack",
"p_system",
"network_peak_powerperrack",
"width",
#"coef_maintenance",
"mttf_switchport",
"p_chip",
"useddistance",
"p_other_idle",
"c_dram",
"k_modulesperblade",
"p_chip_idle",
"p_disk",
"mttf_disk",
"n_srvmodulesreq",
"c_disk",
"chip_mttf",
"p_other",
"height",
"n_disk",
"c_chip",
"n_dram",
"n_switchport",
"p_disk_idle",
"nb_time_step",
);


#check optional parameters
my %dc_tmp=%{$ptr_dc};
if(defined($dc_tmp{cost_of_money}))
{
	push(@list_parameters,"cost_of_money");
}


######################################################
#
# sensitivity exploration
#
######################################################

print(OUTPUTFILE "Name");
foreach my $val (@sensitivity_values){print(OUTPUTFILE "\t$val"); }
print(OUTPUTFILE "\n");

foreach my $param(@list_parameters)
{
	print(OUTPUTFILE "$param");

	foreach my $val(@sensitivity_values)
	{
		my %dc=%{$ptr_dc};
		my %resource=%{$ptr_resource};
		
		if(defined($dc{$param}))
		{
			if($param eq "PUE")
			{
				$dc{$param}=($dc{$param}-1)*$val+1;
			}
			else
			{
				$dc{$param}=$dc{$param}*$val;
			}
		}
		elsif(defined($resource{$param}))
		{
			if($param eq "SPUE")
			{
				$resource{$param}=($resource{$param}-1)*$val+1;
			}
			else
			{
				$resource{$param}=$resource{$param}*$val;
			}
		}
		else{next;}#an optional parameter is not defined in the configuration files

		$helper->createParameterFile("$currentdir/dc.params",\%dc);
		$helper->createParameterFile("$currentdir/s.params",\%resource);
		
		$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/$param.$val");
		
		my $ptr_res_current=$helper->parserResultKernel("$currentdir/output/$param.$val");
		my %res_current=%{$ptr_res_current};
		
		if(!defined($res_current{$output})){print ("error with '$param'\n");exit(0);}
		
		my $current=$res_current{$output}/$ref;

		print(OUTPUTFILE "\t$current");
	}
	print(OUTPUTFILE "\n");
}


close(OUTPUTFILE);

#keep the output file in the current directory
copy("$currentdir/output/$filename.txt","$currentdir/$filename.txt") or die "Copy failed: $!";

####################################################
# gnulpot call
####################################################
if($gnuplot==1)
{
	$helper->makeGnuplotGroupCurve("$currentdir/$filename.txt","$currentdir","$filename","$title",scalar(@sensitivity_values));
}

####################################################
# clean
####################################################
$helper->deleteFile("$currentdir/dc.params");
$helper->deleteFile("$currentdir/s.params");
$helper->deleteFileInDir("$currentdir/output/");

